# @Time     : 2021/5/28
# @Project  : f1z1-g
# @IDE      : PyCharm
# @Author   : Angel
# @Email    : 376355670@qq.com
from .src.runner.base import AbstractRunner
from .src.runner.arunner import AsyncRunner

from .src.api import is_runner, create_runner, start



# @Time     : 2021/6/1
# @Project  : f1z1-g
# @IDE      : PyCharm
# @Author   : Angel
# @Email    : 376355670@qq.com
from abc import ABCMeta, abstractmethod

from common.f1z1_common import ArgsTypes, CoroOrFunction, KwargsTypes, ReturnType


class AbstractRunner(metaclass=ABCMeta):

    def __init__(self, coro_or_func: CoroOrFunction, args: ArgsTypes = None, kwargs: KwargsTypes = None):
        self._coro_or_func = coro_or_func
        self._args = () if not args else kwargs
        self._kwargs = {} if not kwargs else kwargs

    @property
    def coro_or_func(self):
        return self._coro_or_func

    @abstractmethod
    def run(self) -> ReturnType:
        raise NotImplementedError("NotImplemented .run() -> ReturnType")



# @Time     : 2021/6/1
# @Project  : f1z1-g
# @IDE      : PyCharm
# @Author   : Angel
# @Email    : 376355670@qq.com
from asyncio import run

from common.f1z1_common import Executor

from .base import AbstractRunner


class AsyncRunner(AbstractRunner):

    def __init__(self,
                 coro_or_func,
                 args=None,
                 kwargs=None,
                 max_workers: int = None):
        super().__init__(coro_or_func, args, kwargs)
        self._max_workers = max_workers

    def run(self):
        return run(self.main())

    async def main(self):
        executor = Executor(
            coro_or_func=self.coro_or_func,
            args=self._args,
            kwargs=self._kwargs,
            max_workers=self._max_workers
        )
        return await executor.execute()



# @Time     : 2021/5/28
# @Project  : f1z1-g
# @IDE      : PyCharm
# @Author   : Angel
# @Email    : 376355670@qq.com
from .runner.base import AbstractRunner
from .runner import AsyncRunner


def is_runner(value):
    return isinstance(value, AbstractRunner)


def create_runner(coro_or_func,
                  args=None,
                  kwargs=None):
    """
    runner factory
    :param coro_or_func: coroutine, async function or function
    :param args: tuple or None
    :param kwargs: kwargs or None
    :return:
    """
    return AsyncRunner(
        coro_or_func,
        args=args,
        kwargs=kwargs
    )


def start(runner: AbstractRunner):
    if not is_runner(runner):
        raise ValueError(f"runner need IRunner instance, but got {type(runner).__name__}")
    return runner.run()
